﻿using Microsoft.Extensions.Logging;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDataModels.Enums;

namespace SushiBarBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        public OrderLogic(ILogger logger, IOrderStorage orderStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
        }
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() :
                _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            {
                _logger.LogWarning("Wrong Order Status");
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            model.Status = OrderStatus.Принят;
            if (_orderStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool TakeOrderInWork(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Принят)
            {
                _logger.LogWarning("Wrong Order Status");
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            model.Status = OrderStatus.Выполняется;
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool DeliveryOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Выполняется)
            {
                _logger.LogWarning("Wrong Order Status");
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            model.Status = OrderStatus.Готов;
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Готов)
            {
                _logger.LogWarning("Wrong Order Status");
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            model.Status = OrderStatus.Выдан;
            if (_orderStorage.Update(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.DateImplement != null && model.DateCreate > model.DateImplement)
            {
                throw new InvalidOperationException("Заказ не может быть начат после его завершения");
            }
            if (model.Count <= 0)
            {
                throw new ArgumentException("Количество продукта должно быть бльше 0");
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
        }
    }
}
